==> caiviar-0.3.3.tar.gz: caiviar-0.3.3/server/ClientSocket.cpp <==
/* ClientSocket.cpp

java-like implementation of sockets.

Part of the caiviar package. 

Copyright (c) 2002 MobileX AG, http://www.mobilexag.de
Copyright (c) 2002 Peter Dikant <peter.dikant@mobilexag.de>
Copyright (c) 2002 Matthias Kramm <kramm@quiss.org>

This file is distributed under the GPL, see file COPYING for details. */

#include "../config.h"

#if defined(LINUX) || defined(CYGWIN)
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <time.h>
#include <memory.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <asm/types.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <net/if.h>
#include <netdb.h>
#elif defined(WIN32)
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <memory.h>
#ifndef __windows_h_included__
#define __windows_h_included__
#include <windows.h>
#endif
#endif

using namespace std;

#include "ClientSocket.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#if defined(LINUX) || defined(CYGWIN)
static void Sleep(int useconds)
{
	usleep(useconds);
}
static void closesocket(int socket)
{
	close(socket);
}
#endif

/**
 * this constructor is used to initialize the class with an already
 * given socket. This is the situation when a server is listening on
 * 
 */
ClientSocket::ClientSocket(SOCKET client)
{
	this->client = client;
	this->host = NULL;
	this->port = -1;
} 

ClientSocket::ClientSocket(const char *host, int port) {
	this->client = INVALID_SOCKET;
	this->host = (char*)host;
	this->port = port;
}

/**
 * tidy up and close the client socket connection.
 */
ClientSocket::~ClientSocket()
{
	Sleep(500); // wait for slow java clients to close the socket
	closesocket(client);
}

/**
 * read a line from the socket with a predefined buffer size
 */
int ClientSocket::readLine(char *buffer)
{
	return this->readLine(buffer, 1024);
} /* readLine */

int ClientSocket::getBlock(char *buffer, int size)
{
	return recv(client, buffer, size, 
#if defined(LINUX) || defined(CYGWIN)
# ifdef MSG_NOSIGNAL
		MSG_NOSIGNAL
# else
		0x4000
# endif
#else
		0
#endif
		);
}
	


int ClientSocket::isAlive()
{
#ifdef LINUX
    struct sockaddr addr;
    socklen_t len;
    int ret;

    /*memset(&addr,0,sizeof(addr));
    ret = getpeername(client, &addr, &len);
    printf("%d %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x %d %d\n",
	    addr.sa_family,
	    addr.sa_data[0], addr.sa_data[1],
	    addr.sa_data[2], addr.sa_data[3],
	    addr.sa_data[4], addr.sa_data[5],
	    addr.sa_data[6], addr.sa_data[7],
	    addr.sa_data[8], addr.sa_data[9],
	    addr.sa_data[10], addr.sa_data[11],
	    addr.sa_data[12], addr.sa_data[13],
	    ret, errno);*/
    char buffer[16];
    ret = recv(client, buffer, 1, 
#if defined(MSG_NOSIGNAL) && defined(MSG_DONTWAIT) && defined(MSG_PEEK)
	    MSG_PEEK|MSG_DONTWAIT|MSG_NOSIGNAL
#else
	    0x442
#endif
	    );
    printf("%d %d\n", ret, errno);
#endif
    return 1;
}

/**
 * this method will read characters from the socket connection until a newline
 * has been reached. the method will only terminate on newline or buffer full.
 */
int ClientSocket::readLine(char *buffer, int bufferSize)
{
	int pos = 0;
	char lastwascr=0;

	while (pos < bufferSize) {
	
		int ret = recv(client, buffer, 1, 
#if defined(LINUX)
# ifdef MSG_NOSIGNAL
		MSG_NOSIGNAL
# else
		0x4000
# endif
#else
			0
#endif
			);
		if (ret <= 0) { 
#if defined(WIN32)
			if (WSAGetLastError() != 0) {
				fprintf(stderr, "Error reading from socket\n"); //WSAGetLastError()
			}
#elif defined(LINUX) || defined(CYGWIN)
			printf ("Error reading from socket: ret=%d errno=%d\n", ret, errno);
			perror ("perror");
#endif
			*buffer = '\0';
			return 0;
		}

		if (*buffer == '\n') {
			*buffer = '\0'; // get rid of LF
			if (lastwascr) { //get rid of optional CR
				*--buffer = '\0';
				lastwascr = 0;
			}
			return 1;
		} else {
			lastwascr = (*buffer == '\r');
		}
		buffer++;
		pos++;
	}
	return 1;
} /* readLine */

/**
 * print a line of text to the socket connection.
 */
int ClientSocket::print(const char *line)
{
/*	printf(">%s", line);
	if(line[strlen(line)-1]!='\n')
	  printf("\n");*/
#if defined(LINUX) || defined(CYGWIN)
	//write(client, line, strlen(line));
	int ret = send(client, line, strlen(line), 0);
	if (ret <= 0) {
		perror("send returned:");
		printf("send error: %d\n", ret);
		return -1;
	}
#elif defined(WIN32)
	if (send(client, line, strlen(line), 0) == SOCKET_ERROR) {
		fprintf(stderr, "Error writing to socket\n"); //WSAGetLastError()
		return -1;
	}
#endif
    return 0;
} /* print */

/**
 * this a print method that will accept a list of arguments to be included in the
 * output line, similar to the normal printf statement
 */
int ClientSocket::myprintf(const char *line,...)
{
	char buffer[1024];
	va_list arglist;
	va_start(arglist, line);
	vsprintf(buffer, line, arglist);
	va_end(arglist);
	return print(buffer);
} /* printf */

bool ClientSocket::establishConnection()
{
#if defined(LINUX) || defined(CYGWIN)
	int i, sock = -1, ret;
	char buf_ip[100];
	struct hostent *he; 
	unsigned char ip1;
	unsigned char ip2;
	unsigned char ip3;
	unsigned char ip4;
	struct sockaddr_in sin;
	int timeout = 120;

	he = gethostbyname (host);
	if (!he)
	  {
	    printf ("Couldn't resolve %s\n", host);
	    fflush(stdout);
	    return false;
	  }

	ip1 = he->h_addr_list[0][0];
	ip2 = he->h_addr_list[0][1];
	ip3 = he->h_addr_list[0][2];
	ip4 = he->h_addr_list[0][3];
	sin.sin_family = AF_INET;
	sin.sin_port = (port << 8) + (port >> 8);
	memcpy (&sin.sin_addr.s_addr, he->h_addr_list[0], 4);

	printf ("Connecting to %d.%d.%d.%d:%d...\n", ip1, ip2, ip3, ip4, port);
	fflush(stdout);

	sock = socket (AF_INET, SOCK_STREAM, 6);
	if (sock < 0)
	  {
	    fprintf(stderr, "socket (AF_INET, SOCK_STREAM, 6) failed\n");
	    fflush(stderr);
	    return false ;
	  }

	ret = connect (sock, (struct sockaddr *) &sin, sizeof (struct sockaddr_in));
	if (ret < 0)
	  {
	    fprintf(stderr, "connect (socket, sin, len) failed\n");
	    fflush(stderr);
	    return false;
	  }

	client = sock;
	printf("port is %d\n", client);
	return true;
#elif defined(WIN32)
	WSAData wsaData;
	int nRet;

	// Start Winsock up
	nRet = WSAStartup(MAKEWORD(2, 2), &wsaData);
	if (nRet != 0) {
		fprintf(stderr, "WSAStartup() returned error code\n");
		return false;
	}

	if (client != INVALID_SOCKET) {
		return false;
	}

	// convert host address to useable format
	u_long remoteAddr = inet_addr(host);
	if (remoteAddr == INADDR_NONE) {
		// pcHost isn't a dotted IP, so resolve it through DNS
        hostent* he = gethostbyname(host);
		if (he == 0) {
			fprintf(stderr, "Could not resolve remote server address: %s\n" ,host);
			return false;
		}
		remoteAddr = *((u_long*)he->h_addr_list[0]);
	}
	
	// open socket connection
	client = socket(AF_INET, SOCK_STREAM, 0);
	if (client == INVALID_SOCKET) {
		fprintf(stderr, "Could not create client socket\n");
		return false;
	}

	sockaddr_in sinRemote;
	sinRemote.sin_family = AF_INET;
	sinRemote.sin_addr.s_addr = remoteAddr;
	sinRemote.sin_port = htons(port);
	if (connect(client, (sockaddr*)&sinRemote, sizeof(sockaddr_in)) == SOCKET_ERROR) {
		fprintf(stderr, "Could not connect to server\n"); // WSAGetLastError()
		return false;
	}
	return true;
#endif
} /* establishConnection */

==> caiviar-0.3.4.tar.gz: caiviar-0.3.4/server/ClientSocket.cpp <==
/* ClientSocket.cpp

java-like implementation of sockets.

Part of the caiviar package. 

Copyright (c) 2002 MobileX AG, http://www.mobilexag.de
Copyright (c) 2002 Peter Dikant <peter.dikant@mobilexag.de>
Copyright (c) 2002 Matthias Kramm <kramm@quiss.org>

This file is distributed under the GPL, see file COPYING for details. */

#include "../config.h"

#if defined(LINUX) || defined(CYGWIN)
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <time.h>
#include <memory.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <asm/types.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <net/if.h>
#include <netdb.h>
#elif defined(WIN32)
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <memory.h>
#ifndef __windows_h_included__
#define __windows_h_included__
#include <windows.h>
#endif
#endif

#include "ClientSocket.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#if defined(LINUX) || defined(CYGWIN)
static void Sleep(int useconds)
{
	usleep(useconds);
}
static void closesocket(int socket)
{
	close(socket);
}
#endif

/**
 * this constructor is used to initialize the class with an already
 * given socket. This is the situation when a server is listening on
 * 
 */
ClientSocket::ClientSocket(SOCKET client)
{
	this->client = client;
	this->host = NULL;
	this->port = -1;
} 

ClientSocket::ClientSocket(const char *host, int port) {
	this->client = INVALID_SOCKET;
	this->host = (char*)host;
	this->port = port;
}

/**
 * tidy up and close the client socket connection.
 */
ClientSocket::~ClientSocket()
{
	Sleep(500); // wait for slow java clients to close the socket
	closesocket(client);
}

/**
 * read a line from the socket with a predefined buffer size
 */
int ClientSocket::readLine(char *buffer)
{
	return this->readLine(buffer, 1024);
} /* readLine */

int ClientSocket::getBlock(char *buffer, int size)
{
	return recv(client, buffer, size, 
#if defined(LINUX) || defined(CYGWIN)
# ifdef MSG_NOSIGNAL
		MSG_NOSIGNAL
# else
		0x4000
# endif
#else
		0
#endif
		);
}
	


int ClientSocket::isAlive()
{
#ifdef LINUX
    struct sockaddr addr;
    socklen_t len;
    int ret;

    /*memset(&addr,0,sizeof(addr));
    ret = getpeername(client, &addr, &len);
    printf("%d %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x %d %d\n",
	    addr.sa_family,
	    addr.sa_data[0], addr.sa_data[1],
	    addr.sa_data[2], addr.sa_data[3],
	    addr.sa_data[4], addr.sa_data[5],
	    addr.sa_data[6], addr.sa_data[7],
	    addr.sa_data[8], addr.sa_data[9],
	    addr.sa_data[10], addr.sa_data[11],
	    addr.sa_data[12], addr.sa_data[13],
	    ret, errno);*/
    char buffer[16];
    ret = recv(client, buffer, 1, 
#if defined(MSG_NOSIGNAL) && defined(MSG_DONTWAIT) && defined(MSG_PEEK)
	    MSG_PEEK|MSG_DONTWAIT|MSG_NOSIGNAL
#else
	    0x442
#endif
	    );
    printf("%d %d\n", ret, errno);
#endif
    return 1;
}

/**
 * this method will read characters from the socket connection until a newline
 * has been reached. the method will only terminate on newline or buffer full.
 */
int ClientSocket::readLine(char *buffer, int bufferSize)
{
	int pos = 0;
	char lastwascr=0;

	while (pos < bufferSize) {
	
		int ret = recv(client, buffer, 1, 
#if defined(LINUX)
# ifdef MSG_NOSIGNAL
		MSG_NOSIGNAL
# else
		0x4000
# endif
#else
			0
#endif
			);
		if (ret <= 0) { 
#if defined(WIN32)
			if (WSAGetLastError() != 0) {
				fprintf(stderr, "Error reading from socket\n"); //WSAGetLastError()
			}
#elif defined(LINUX) || defined(CYGWIN)
			fprintf (stderr, "Error reading from socket: ret=%d errno=%d\n", ret, errno);
			perror ("perror");
#endif
			*buffer = '\0';
			return 0;
		}

		if (*buffer == '\n') {
			*buffer = '\0'; // get rid of LF
			if (lastwascr) { //get rid of optional CR
				*--buffer = '\0';
				lastwascr = 0;
			}
			return 1;
		} else {
			lastwascr = (*buffer == '\r');
		}
		buffer++;
		pos++;
	}
	return 1;
} /* readLine */

/**
 * print a line of text to the socket connection.
 */
int ClientSocket::print(const char *line)
{
/*	printf(">%s", line);
	if(line[strlen(line)-1]!='\n')
	  printf("\n");*/
#if defined(LINUX) || defined(CYGWIN)
	//write(client, line, strlen(line));
	int ret = send(client, line, strlen(line), 0);
	if (ret <= 0) {
		perror("send returned:");
		fprintf(stderr, "send error: %d\n", ret);
		return -1;
	}
#elif defined(WIN32)
	if (send(client, line, strlen(line), 0) == SOCKET_ERROR) {
		fprintf(stderr, "Error writing to socket\n"); //WSAGetLastError()
		return -1;
	}
#endif
    return 0;
} /* print */

/**
 * this a print method that will accept a list of arguments to be included in the
 * output line, similar to the normal printf statement
 */
int ClientSocket::myprintf(const char *line,...)
{
	char buffer[1024];
	va_list arglist;
	va_start(arglist, line);
	vsprintf(buffer, line, arglist);
	va_end(arglist);
	return print(buffer);
} /* printf */

bool ClientSocket::establishConnection()
{
#if defined(LINUX) || defined(CYGWIN)
	int i, sock = -1, ret;
	char buf_ip[100];
	struct hostent *he; 
	unsigned char ip1;
	unsigned char ip2;
	unsigned char ip3;
	unsigned char ip4;
	struct sockaddr_in sin;
	int timeout = 120;

	he = gethostbyname (host);
	if (!he)
	  {
	    printf ("Couldn't resolve %s\n", host);
	    fflush(stdout);
	    return false;
	  }

	ip1 = he->h_addr_list[0][0];
	ip2 = he->h_addr_list[0][1];
	ip3 = he->h_addr_list[0][2];
	ip4 = he->h_addr_list[0][3];
	sin.sin_family = AF_INET;
	sin.sin_port = (port << 8) + (port >> 8);
	memcpy (&sin.sin_addr.s_addr, he->h_addr_list[0], 4);

	printf ("Connecting to %d.%d.%d.%d:%d...\n", ip1, ip2, ip3, ip4, port);
	fflush(stdout);

	sock = socket (AF_INET, SOCK_STREAM, 6);
	if (sock < 0)
	  {
	    fprintf(stderr, "socket (AF_INET, SOCK_STREAM, 6) failed\n");
	    fflush(stderr);
	    return false ;
	  }

	ret = connect (sock, (struct sockaddr *) &sin, sizeof (struct sockaddr_in));
	if (ret < 0)
	  {
	    fprintf(stderr, "connect (socket, sin, len) failed\n");
	    fflush(stderr);
	    return false;
	  }

	client = sock;
	printf("port is %d\n", client);
	return true;
#elif defined(WIN32)
	WSAData wsaData;
	int nRet;

	// Start Winsock up
	nRet = WSAStartup(MAKEWORD(2, 2), &wsaData);
	if (nRet != 0) {
		fprintf(stderr, "WSAStartup() returned error code\n");
		return false;
	}

	if (client != INVALID_SOCKET) {
		return false;
	}

	// convert host address to useable format
	u_long remoteAddr = inet_addr(host);
	if (remoteAddr == INADDR_NONE) {
		// pcHost isn't a dotted IP, so resolve it through DNS
        hostent* he = gethostbyname(host);
		if (he == 0) {
			fprintf(stderr, "Could not resolve remote server address: %s\n" ,host);
			return false;
		}
		remoteAddr = *((u_long*)he->h_addr_list[0]);
	}
	
	// open socket connection
	client = socket(AF_INET, SOCK_STREAM, 0);
	if (client == INVALID_SOCKET) {
		fprintf(stderr, "Could not create client socket\n");
		return false;
	}

	sockaddr_in sinRemote;
	sinRemote.sin_family = AF_INET;
	sinRemote.sin_addr.s_addr = remoteAddr;
	sinRemote.sin_port = htons(port);
	if (connect(client, (sockaddr*)&sinRemote, sizeof(sockaddr_in)) == SOCKET_ERROR) {
		fprintf(stderr, "Could not connect to server\n"); // WSAGetLastError()
		return false;
	}
	return true;
#endif
} /* establishConnection */
